/*	injection_isrs.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "9s12xdp512.h"
#include "interrupts.h"

/* Summary of intended injection timing scheme
 * 
 * Set ECT timer based on a reference point (probably single cam trigger or optionally toggle bit based on missing tooth on crank)
 * Set time to open for each channel based on the nearest engine position information and the current value of the timer at that signal (a particular edge of a particular crank tooth)
 * Injector turns on and generates interrupt, period of injection in timer units calculated from period of injection and injection start time.
 * Injector turns off and generates an interrupt, ignore that interrupt
 * Repeat this for all injectors each engine cycle
 */

/* Each interrupt handler should :
 * 
 * 1,2	Clear its interrupt flag, Disable all interrupts (or vice versa?)
 * 3	Check to see if its just opened or just closed
 * 4	If just opened, set the period of the cycle and the action to switch off
 * 5	If just closed set the action to switch on (and maybe clean up?)
 * 6	Return
 */

// Current

void Injector1ISR(void)
{
	PTT ^= 0x04;
}

void Injector2ISR(void)
{
	PTT ^= 0x08;
}

void Injector3ISR(void)
{
	PTT ^= 0x10;
}

void Injector4ISR(void)
{
	PTT ^= 0x20;
}

void Injector5ISR(void)
{
	PTT ^= 0x40;
}

void Injector6ISR(void)
{
	PTT ^= 0x80;
}

